# -*- coding: utf-8 -*-

"""
MIT License

Copyright (c) 2019 Arthurdw

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import requests, json
from .errors import *
from .utils import get_Oxford

class Dictionary:
    """Represents a dictionary from the Oxford Dictionaries, 
    This class connects to the Oxford Dictionaries API and receives information.
    
    Arguments for :class:`Dictionary`:
    
    Parameters
    ----------
    app_id: Required[:class:`str`]  
        The application id is used to connect to Oxford Dictionaries.  
        Receive your application id here:  
        https://developer.oxforddictionaries.com/  
    app_key: Required[:class:`str`]  
        The application key is used to connect to Oxford Dictionaries.  
        Receive your application key here:  
        https://developer.oxforddictionaries.com/  
    language: Optional[:class:`str`]  
        The language Pythonary will search in.  
        Default language is 'EN' or English.  
        This variable must be the 'Language Code' and not the full language name!  
        See all the language's here:  
        https://developer.oxforddictionaries.com/documentation/languages  
    """
    def __init__(self, app_id, app_key, language=None):
        self.id = app_id
        self.key = app_key
        if language is None: self.language = 'en'
        else: self.language = language
        self.word = None
        self.json = None

    def get_app_id(self):
        if self.id == None: raise MissingCoreArguments(f'ERROR 003 | Missing the `app_id` variable, please check your setup!')
        else: return self.id

    def get_app_key(self):
        if self.key == None: raise MissingCoreArguments(f'ERROR 002 | Missing the `app_key` variable, please check your setup!')
        else: return self.key

    def get_app_language(self):
        language = self.language.lower()
        country_list = ['en-gb', 'en-us', 'en', 'e', 'sm', 'ss', 'wt', 'nn', 'so', 'lv', 'id', 'ur', 'zu', 'ro', 'hi', 'de', 'pt', 'ta', 'gu', 'qu', 'el', 'xh', 'tt', 'te', 'tpi', 'tk', 'tg', 'zh']
        if language not in country_list: raise MissingCoreArguments(f'ERROR 004 | The `language` variable doesn\'t match the allowed keys, please check your setup!')
        else: return self.language

    def Select(self, word):
        if word == None: raise SyntaxError(f'ERROR 010 | Missing the `word` variable in `Select`!')
        else: 
            self.word = word
            json = get_Oxford(word=word, language=Dictionary.get_app_language(self), app_id=Dictionary.get_app_id(self), app_key=Dictionary.get_app_key(self))
            self.json = json

    def parsed(self):
        """Retrieve the parsed JSON data yourself.
        Returns :class:`dict`"""
        return json.loads(self.json)
    
    def name(self):
        """Retrieve the name from the selected word!
        Returns :class:`str`"""
        return Dictionary.parsed(self)["results"][0]['lexicalEntries'][0]['text']
    
    def etymologies(self):
        """Retrieve the etymologies from the selected word
        Returns :class:`str`"""
        return Dictionary.parsed(self)["results"][0]['lexicalEntries'][0]['entries'][0]['etymologies'][0]
    
    def definition(self):
        """Retrieve the definition from the selected word
        Returns :class:`str`"""
        return Dictionary.parsed(self)["results"][0]['lexicalEntries'][0]['entries'][0]['senses'][0]['definitions'][0]

    def short_definition(self):
        """Retrieve a shorter definition from the selected word
        Returns :class:`str`"""
        return Dictionary.parsed(self)["results"][0]['lexicalEntries'][0]['entries'][0]['senses'][0]['shortDefinitions'][0]
    
    def example(self):
        """Retrieves an example sentence with the selected word in it.
        Returns :class:`str`"""
        return Dictionary.parsed(self)["results"][0]['lexicalEntries'][0]['entries'][0]['senses'][0]['examples'][0]['text']

    def sub_definition(self):
        """Retrieve a sub definition from the selected word
        Returns :class:`str`"""
        return Dictionary.parsed(self)["results"][0]['lexicalEntries'][0]['entries'][0]['senses'][0]['subsenses'][0]['definitions'][0]

    def sub_short_definition(self):
        """Retrieve a sub short definition from the selected word
        Returns :class:`str`"""
        return Dictionary.parsed(self)["results"][0]['lexicalEntries'][0]['entries'][0]['senses'][0]['subsenses'][0]['shortDefinitions'][0]

    def sub_example(self):
        """Retrieve a sub example from the selected word
        Returns :class:`str`"""
        return Dictionary.parsed(self)["results"][0]['lexicalEntries'][0]['entries'][0]['senses'][0]['subsenses'][0]['examples'][0]['text']
    
    def language_code(self):
        """Retrieve the language code from the selected word
        Returns :class:`str`"""
        return Dictionary.parsed(self)["results"][0]['lexicalEntries'][0]['language']

    def category(self):
        """Retrieve the form/category from the selected word (Noun,...)
        Returns :class:`str`"""
        return Dictionary.parsed(self)["results"][0]['lexicalEntries'][0]['lexicalCategory']['id']
    
    def audiofile(self):
        """Retrieve the audiofile download url from the selected word.
        Returns :class:`str`"""
        return Dictionary.parsed(self)["results"][0]['lexicalEntries'][0]['pronunciations'][0]['audioFile']
    
    def audiofile_dialect(self):
        """Retrieve the audiofile dialect from the selected word.
        Returns :class:`str`"""
        return Dictionary.parsed(self)["results"][0]['lexicalEntries'][0]['pronunciations'][0]['dialects'][0]

    def phonetic(self):
        """Retrieve the phonetic spelling from the selected word.
        Returns :class:`str`"""
        return Dictionary.parsed(self)["results"][0]['lexicalEntries'][0]['pronunciations'][0]['phoneticSpelling']

    def phonetic_notation(self):
        """Retrieve notation from this word its `phonetic` notation
        Returns :class:`str`"""
        return Dictionary.parsed(self)["results"][0]['lexicalEntries'][0]['pronunciations'][0]['phoneticNotation']
